/*
    Copyright (C) 2005 - 2009 Manuel Lausch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Manuel Lausch
    Salierstr. 52
    75177 Pforzheim
    GERMANY
    software@manuellausch.de
*/

#include <iostream>
#include <stdlib.h>
#include <string>
#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_ttf.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include "snake.h"
#include "config.h"

using std::cerr;
using std::endl;
using std::cout;
using std::string;



SDL_Surface * initMyGame(config*);
void help();

int main(int argc, char** argv)
{
	config *konfiguration = new config(); // Erstellen einer grundkonfiguration
	 // max amount of players, it depends on the defined colors and keys

	srand( (unsigned) time(NULL) ) ;

	int ende, taste = 0, num_player = 1;
	int LevelNumberVonArgv = 0; // Von wem kommt nur diese Variablenbezeichnung? Sie stellt jedenfalls dar,
	// an welchem index in argv der dateiname des levels steht.

	// Diese Routine wertet die Commandlineargumente aus. Der code ist dirty sollte mal ersetzt werden
	for(int i = 1; i < argc; i++)
	{
		if(argv[i][0] == '-')
		{
			switch(argv[i][1])
			{
				case 'P':
					num_player = atoi(argv[i + 1]); // sowas fhrt gerne zu seg-faults, wenn keine zahl eingegeben wurde
					break;

				case 'L':
					LevelNumberVonArgv = i + 1; // Und das ist hnlich schlimm
					break;

				case 'h':
				case '?':
					help();
					return 0;
					break;
			}
		}
	}
	if((int)konfiguration->get_maxplayer() < num_player)
	{
		cout << "Es koennen maximal " << konfiguration->get_maxplayer() << " Spieler mitspielen." << endl;
		return 0;
	}
	if(num_player == 0)
	{
		cout << "Wie bitte soll man ohne Spieler spielen???" << endl;
		return 0;
	}
	if(num_player < 0)
	{
		cout << "Wenn du mir verrtst, wie ein negativer Spieler ausschaut, kann ich den gerne einbauen..." << endl;
		return 0;
	}

	SDL_Surface *pDisplay, *pBackup = 0, *pPause = 0;
	SDL_Rect drec;
	SDL_Event event;
	snake *player[num_player];
	int pause = 0;
	SDL_TimerID timer[num_player];

	// Variablen fr Soundausgabe
	Mix_Music *music;
	Mix_Chunk *punkt, *crash;

	// Audioausgabe starten
	Mix_OpenAudio(44100, AUDIO_S16, 2, 1024);
	punkt = Mix_LoadWAV(konfiguration->get_filepath("click.wav").c_str());
	crash = Mix_LoadWAV(konfiguration->get_filepath("crash.wav").c_str());
	music = Mix_LoadMUS(konfiguration->get_filepath("ein1.mod").c_str());

	Mix_AllocateChannels(2);

	// Lautstrken setzen
	Mix_VolumeChunk(crash, MIX_MAX_VOLUME);
	Mix_VolumeChunk(punkt, MIX_MAX_VOLUME/3*2);

	// Backgroundmusic starten
	Mix_PlayMusic(music, -1); // Endlosschleife fr die Musik: Problem bei neustart der musik wird die lautstrke wieder auf max gesetzt
	Mix_VolumeMusic(MIX_MAX_VOLUME/3); // Lautstrke kann erst nach Mix_PlayMusic gendert werden


	// init playground
	pDisplay = initMyGame(konfiguration);
	playground *pPlayground;
	if(LevelNumberVonArgv != 0) // ist ein Levelname definiert ?
		pPlayground = new playground(pDisplay, argv[LevelNumberVonArgv], konfiguration); // Ja

	else
		pPlayground = new playground(pDisplay, konfiguration); // Nein

	pPause = IMG_Load(konfiguration->get_filepath("pause.png").c_str());

	// also wenn ich schon so ne definition wie
	// SDL_Surface *SDL_CreateRGBSurface (Uint32 flags, int breite, int hhe, int bpp, Uint32 Rmaske, Uint32 Gmaske, int32 Bmaske, Uint32 Amask)
	// sehe, dann doch lieber so ;)
	pBackup = IMG_Load(konfiguration->get_filepath("pause.png").c_str());

	drec.w = pPause->w;
	drec.h = pPause->h;
	drec.x = (konfiguration->get_feldbreite() * konfiguration->get_quadsize() - pPause->w) / 2;
	drec.y = (konfiguration->get_feldhohe() * konfiguration->get_quadsize() - pPause->h) / 2;

	// create players
	for(int i = 0; i < num_player; i++)
	{
		player[i] = new snake(pDisplay, pPlayground, konfiguration);
		player[i]->set_speed(konfiguration->get_player(i)->get_playerspeed());
		player[i]->set_color(konfiguration->get_player(i)->get_playercolor());
		timer[i] = SDL_AddTimer(konfiguration->get_player(i)->get_playerspeed(), snake::step_timer, player[i]);
	}

	ende = 0;
	SDL_EventState(SDL_KEYUP, SDL_IGNORE); // Wir wollen nur Tastendrcke verarbeiten, loslassen der taste interessiert nicht.
	do // Event-loop
	{
		SDL_WaitEvent(&event);
		switch(event.type)
		{
			case SDL_KEYDOWN:
				taste = event.key.keysym.sym;
				break;
			case SDL_QUIT:
				ende += 1;
				break;

			case SDL_USEREVENT:
				if(event.user.code == 'p')
					Mix_PlayChannel(-1, punkt, 0);
				if(event.user.code == 'c')
					Mix_PlayChannel(-1, crash, 0);
				break;
			default:
				taste = 0;
		}

		// Pause Funktion
		if (taste == ' ')
		{
			switch(pause)
			{
			case 0: // pause aktivieren
				pause = 1;
				snake::pause = true;
				SDL_BlitSurface(pDisplay, &drec, pBackup, NULL);
				SDL_BlitSurface(pPause, NULL, pDisplay, &drec);
				SDL_Flip(pDisplay);
				break;
			case 1: // pause beenden
				pause = 0;
				SDL_BlitSurface(pBackup, NULL, pDisplay, &drec);
				SDL_Flip(pDisplay);
				snake::pause = false;
				break;
			}
		}
		// ! Hier sollten dann aber auch alle weiter bettigten Tasten ignoeriert werden.
		if (pause == 1) continue; 	// was hiermit quick & dirty durch abbruch weiterer
								// verarbeitung im pause mode geschehen ist :)

		for(int i=0; i<num_player; i++)
		{
			if(taste == konfiguration->get_player(i)->get_key(up))			player[i]->ways.push(up);
			else if(taste == konfiguration->get_player(i)->get_key(down)) 	player[i]->ways.push(down);
			else if(taste == konfiguration->get_player(i)->get_key(right)) 	player[i]->ways.push(right);
			else if(taste == konfiguration->get_player(i)->get_key(left)) 	player[i]->ways.push(left);
		}

	}
	while(0 == ende);

	// cleaning the memory and quit the game
	for(int i = 0; i < num_player; i++)
	{
		SDL_RemoveTimer(timer[i]);
		delete player[i];
	}

	delete pPlayground;
	konfiguration->save();
	delete konfiguration;

	return 0;
}

void help()
{
	cout << "snake -L [Level (Dateiname)] -P [Anzahl Spieler]\n\n" << endl;
	cout << "Befehl:\t\tFunktion:\n\n";
	cout << "-L\t\tLevel (Dateiname)" << endl;
	cout << "-P\t\tSpieleranzahl" << endl;
	cout << "-h\t\tDiese Hilfe" << endl;
	cout << "-?\t\tSiehe -H" << endl;
}

SDL_Surface * initMyGame(config *konfiguration)
{
	SDL_Surface * pDisplay;

	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER|SDL_HWPALETTE) < 0)
	{
	cerr << "SDL Konnte nicht initialisiert werden:  " << SDL_GetError() << endl;
		exit(-1);
	}

	if(TTF_Init() < 0)
	{
	cerr << "TTF konnte nicht initialisiert werden:  " << TTF_GetError() << endl;
		exit(-1);
	}

	atexit(SDL_Quit);
	atexit(TTF_Quit);

	if (!(pDisplay = SDL_SetVideoMode(konfiguration->get_quadsize() * konfiguration->get_feldbreite(), konfiguration->get_quadsize() * konfiguration->get_feldhohe() + 50, 16, SDL_HWSURFACE|SDL_HWACCEL|SDL_HWPALETTE)))
	{
	cerr << "Video Mode konnte nicht initialisiert werden:  " << SDL_GetError() << endl;
		exit(-1);
	}

	hlineColor(pDisplay, 0, konfiguration->get_quadsize() * konfiguration->get_feldbreite(), konfiguration->get_quadsize() * konfiguration->get_feldhohe(), 0xffffffff);
	SDL_Flip(pDisplay);

	SDL_WM_SetCaption((string("Snake ") + konfiguration->get_version()).c_str(), NULL);

	SDL_WM_SetIcon(SDL_LoadBMP(konfiguration->get_filepath("icon.bmp").c_str()), NULL);
	SDL_ShowCursor(SDL_DISABLE);


	return pDisplay;
}
