/*
    Copyright (C) 2005 - 2009 Manuel Lausch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Manuel Lausch
    Salierstr. 52
    75177 Pforzheim
    GERMANY
    software@manuellausch.de
*/

#include "playground.h"
#include "config.h"
#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include "definitions.h"
#include <fstream>
#include <iostream>

playground::playground(SDL_Surface* pDisp, config *konfiguration)
{
	this->konfiguration = konfiguration;
	initparts();
	pDisplay = pDisp;
	grid = new field[konfiguration->get_feldgrose()];

	for(unsigned int x = 0; x < konfiguration->get_feldbreite(); x++)
		for(unsigned int y = 0; y < konfiguration->get_feldhohe(); y++)
			del_field(x, y);


}

playground::playground(SDL_Surface* pDisp, char* wall_name, config *konfiguration)
{
	this->konfiguration = konfiguration;
	initparts();
	// DIese Funktion ist hoch gefhrlich, da berhaupt keine kontrolle, was fr
	// Daten eingelesen werden.
	grid = new field[konfiguration->get_feldgrose()];
	pDisplay = pDisp;
	std::ifstream fin(konfiguration->get_filepath(wall_name).c_str());
	char ch;

	// hm die statischen werte sind da noch bld. mssten ins level file eingebaut werden
	// genial wre es, wenn die gre des feldes dann entsprechend des levels angepasst wird
	for(int y = 0; y < 50; y++)
		for(int x = 0; x < 50; x++)
		{
			do
			{
				fin.get(ch);
			} while (ch == '\n'); // zeilenumbrche verwerfen

			if(ch == f_empty || ch == ' ')
				del_field(x, y);
			if(ch == f_wall || ch == '#')
				set_field(x, y, f_wall, 0xffffffff, solid);
		}

	fin.close();
}

playground::~playground()
{
	SDL_FreeSurface(pEdge1);
	SDL_FreeSurface(pEdge2);
	SDL_FreeSurface(pEdge3);
	SDL_FreeSurface(pEdge4);
	SDL_FreeSurface(pSolid);
	SDL_FreeSurface(pBlock);
	SDL_FreeSurface(pCircle);
	SDL_FreeSurface(pStart1);
	SDL_FreeSurface(pStart2);
	SDL_FreeSurface(pStart3);
	SDL_FreeSurface(pStart4);
	SDL_FreeSurface(pPartX);
	SDL_FreeSurface(pPartY);
	delete[] grid;
}

void playground::set_field(short int x, short int y, field type, Uint32 color, form design)
{
	SDL_Rect drect;
	SDL_Color color2;

	grid[x+y*konfiguration->get_feldbreite()] = type;
	if(type == f_food)	design=circle;	// food hat geflligst rund zu sein ;)

	drect.x = x * konfiguration->get_quadsize();
	drect.y = y * konfiguration->get_quadsize();
	drect.w = drect.h = konfiguration->get_quadsize();
	rgbsplit(color, color2.r, color2.g, color2.b);
	switch(design)
	{
	case circle:
		SDL_SetColors(pCircle, &color2,255,255);
		SDL_BlitSurface(pCircle, NULL ,pDisplay, &drect);
		break;

	case block:
		SDL_SetColors(pBlock, &color2,255,255);
		SDL_BlitSurface(pBlock, NULL ,pDisplay, &drect);
		break;

	case solid:
		SDL_SetColors(pSolid, &color2,255,255);
		SDL_BlitSurface(pSolid, NULL ,pDisplay, &drect);
		break;

	case partX:
		SDL_SetColors(pPartX, &color2,255,255);
		SDL_BlitSurface(pPartX, NULL ,pDisplay, &drect);
		break;

	case partY:
		SDL_SetColors(pPartY, &color2,255,255);
		SDL_BlitSurface(pPartY, NULL ,pDisplay, &drect);
		break;

	case edge1:
		SDL_SetColors(pEdge1, &color2,255,255);
		SDL_BlitSurface(pEdge1, NULL ,pDisplay, &drect);
		break;

	case edge2:
		SDL_SetColors(pEdge2, &color2,255,255);
		SDL_BlitSurface(pEdge2, NULL ,pDisplay, &drect);
		break;

	case edge3:
		SDL_SetColors(pEdge3, &color2,255,255);
		SDL_BlitSurface(pEdge3, NULL ,pDisplay, &drect);
		break;

	case edge4:
		SDL_SetColors(pEdge4, &color2,255,255);
		SDL_BlitSurface(pEdge4, NULL ,pDisplay, &drect);
		break;

	case begin1:
		SDL_SetColors(pStart1, &color2,255,255);
		SDL_BlitSurface(pStart1, NULL ,pDisplay, &drect);
		break;

	case begin2:
		SDL_SetColors(pStart2, &color2,255,255);
		SDL_BlitSurface(pStart2, NULL ,pDisplay, &drect);
		break;

	case begin3:
		SDL_SetColors(pStart3, &color2,255,255);
		SDL_BlitSurface(pStart3, NULL ,pDisplay, &drect);
		break;

	case begin4:
		SDL_SetColors(pStart4, &color2,255,255);
		SDL_BlitSurface(pStart4, NULL ,pDisplay, &drect);
		break;

	default: ;
		// zum unterdrcken einiger warnungen.
	}


	SDL_UpdateRects(pDisplay, 1, &drect);
}

void playground::del_field(short int x, short int y)
{
	playground::set_field(x,y, f_empty, 0x000000ff, solid);
}

field playground::get_field(short int x, short int y)
{
//	std::cout << grid[x][y] << "  " << x << "  " << y << std::endl;
	return grid[x+y*konfiguration->get_feldbreite()];
}


void playground::rgbsplit(Uint32 color, Uint8& r, Uint8& g, Uint8& b)
{
	r = ((color & 0xff000000) >>24);
	g = ((color & 0x00ff0000) >>16);
	b = ((color & 0x0000ff00) >>8);
}


void playground::initparts()
{
	pPartX  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255);
	pPartY  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255);
	pCircle = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255);
	pSolid  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255);
	pBlock  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255);
	pEdge1  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // links/runter
	pEdge2  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // links/rauf
	pEdge3  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // rechts/rauf
	pEdge4  = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // rechts/runter
	pStart1 = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // rauf
	pStart2 = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // rechts
	pStart3 = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // runter
	pStart4 = SDL_CreateRGBSurface(SDL_HWSURFACE, konfiguration->get_quadsize(), konfiguration->get_quadsize(), 8, 255,255,255,255); // links

	boxColor(pPartX, 0, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 2, 0xffffffff);
	filledCircleColor(pCircle, konfiguration->get_quadsize() / 2 - 1, konfiguration->get_quadsize() / 2 - 1, (konfiguration->get_quadsize() - 1) / 2, 0xffffffff);
	boxColor(pBlock, 0, 0, konfiguration->get_quadsize() - 2, konfiguration->get_quadsize() - 2, 0xffffffff);
	boxColor(pSolid, 0, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 1, 0xffffffff);
	boxColor(pPartY, 0, 0, konfiguration->get_quadsize() - 2, konfiguration->get_quadsize() - 1, 0xffffffff);
	//boxColor(pEdge1, 0, 0, konfiguration->get_quadsize() - 2, konfiguration->get_quadsize() - 1, 0xffffffff);
	//filledTrigonColor(pEdge1, konfiguration->get_quadsize() * 1/4, 0, konfiguration->get_quadsize() - 1, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() * 3/4  , 0x000000ff);
	//boxColor(pEdge2, 0, 0, konfiguration->get_quadsize() - 2, konfiguration->get_quadsize() - 2, 0xffffffff);
	//illedTrigonColor(pEdge2, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() * 1/4,konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() * 1/4, konfiguration->get_quadsize() - 1, 0x000000ff);
	//boxColor(pEdge3, 0, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 2, 0xffffffff);
	//filledTrigonColor(pEdge3, 0, konfiguration->get_quadsize() * 1/4, konfiguration->get_quadsize() * 3/4, konfiguration->get_quadsize() - 1, 0, konfiguration->get_quadsize() - 1, 0x000000ff);
	//boxColor(pEdge4, 0, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 1, 0xffffffff);
	//filledTrigonColor(pEdge4, 0, 0, konfiguration->get_quadsize() * 3/4, 0, 0, konfiguration->get_quadsize() * 3/4, 0x000000ff);
	//pixelColor(pEdge4, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 1, 0x000000ff);

	filledTrigonColor(pStart1, 0, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 2, konfiguration->get_quadsize() - 1, (konfiguration->get_quadsize() / 2) - 1, 0, 0xffffffff);
	filledTrigonColor(pStart2, 0, 0, konfiguration->get_quadsize() - 2, (konfiguration->get_quadsize() / 2) - 1, 0, konfiguration->get_quadsize() - 2, 0xffffffff);
	filledTrigonColor(pStart3, 0, 0, konfiguration->get_quadsize() - 2, 0, (konfiguration->get_quadsize() / 2) - 1, konfiguration->get_quadsize() - 1, 0xffffffff);
	filledTrigonColor(pStart4, 1, (konfiguration->get_quadsize() / 2) - 1, konfiguration->get_quadsize() - 1, 0, konfiguration->get_quadsize() -1, konfiguration->get_quadsize() - 2, 0xffffffff);

	filledCircleColor(pEdge1, 0,konfiguration->get_quadsize()-2,konfiguration->get_quadsize()-2, 0xffffffff);
	filledCircleColor(pEdge2, 0,0, konfiguration->get_quadsize()-2, 0xffffffff);
	filledCircleColor(pEdge3, konfiguration->get_quadsize()-2,0,konfiguration->get_quadsize()-2, 0xffffffff);
	filledCircleColor(pEdge4, konfiguration->get_quadsize()-2, konfiguration->get_quadsize()-2, konfiguration->get_quadsize()-2, 0xffffffff);
	pixelColor(pEdge4, konfiguration->get_quadsize() - 1, konfiguration->get_quadsize() - 1, 0x000000ff);

}
